@@ -8,7 +8,6 @@ from functools import wraps
 import fudge
 import os
 import random
-from unittest import expectedFailure
 
 from ._utils import TestCase
 
@@ -121,6 +120,21 @@ class BaseDonationFormViewTestCase(TestCase):
         self.assertEqual(200, response.status_code, msg="sanity check")
         return response
 
+    def get_fake_post_request(self, confirmed=False):
+        d = {} if not confirmed else {"confirmed": u"1"}
+        return self.factory.post(self.url, d)
+
+    @property
+    def fake_get_request(self):
+        return self.factory.get(self.url)
+
+    def get_post_view(self, confirmed=False):
+        v = views.DonationFormView(confirm=True)
+        v.request = self.get_fake_post_request(confirmed=confirmed)
+        return v
+
+    post_view = property(get_post_view)
+
 
 # TODO: move to armstrong.dev
 def get_response(func):
@@ -135,10 +149,6 @@ class DonationFormViewGetTestCase(BaseDonationFormViewTestCase):
     def test_adds_form_action_url_to_context(self, response):
         self.assert_value_in_context(response, "form_action_url", "")
 
-    @get_response
-    def test_adds_donor_form_to_context(self, response):
-        self.assert_type_in_context(response, "donor_form", forms.DonorForm)
-
     @get_response
     def test_adds_donation_formset_to_context(self, response):
         self.assert_type_in_context(response, "donation_form",
@@ -150,6 +160,93 @@ class DonationFormViewGetTestCase(BaseDonationFormViewTestCase):
         donation_form = view.get_donation_form()
         self.assertIsA(donation_form, forms.CreditCardDonationForm)
 
+    def test_get_context_turns_kwargs_into_params(self):
+        r = lambda: random.randint(100, 200)
+        random_kwargs = {
+            "slug%d" % r(): "foo-%d" % r(),
+        }
+        view = self.get_view_object()
+        context = view.get_context_data(**random_kwargs)
+        self.assertEqual(len(context["params"]), len(random_kwargs),
+                msg="verify context.params is the same length")
+        for key in context["params"].keys():
+            self.assert_(key in random_kwargs)
+
+    def test_form_is_invalid_passes_kwargs_to_get_context_data(self):
+        r = lambda: random.randint(100, 200)
+        random_kwargs = {
+            "slug%d" % r(): "foo-%d" % r(),
+        }
+
+        get_context_data = fudge.Fake()
+        get_context_data.expects_call().with_args(**random_kwargs)
+
+        view = self.post_view
+        with fudge.patched_context(view, "get_context_data", get_context_data):
+            view.post({}, **random_kwargs)
+
+        fudge.verify()
+
+    def test_form_is_valid_passes_kwargs_to_get_context_data(self):
+        donation, donation_form = self.random_donation_and_form
+        r = lambda: random.randint(100, 200)
+        random_kwargs = {
+            "slug%d" % r(): "foo-%d" % r(),
+        }
+
+        get_context_data = fudge.Fake()
+        get_context_data.expects_call().with_args(**random_kwargs)
+
+        view = self.post_view
+        with fudge.patched_context(view, "get_context_data", get_context_data):
+            view.form_is_valid(donation_form, **random_kwargs)
+
+        fudge.verify()
+
+    def test_form_is_valid_passes_kwargs_to_purchase_failed(self):
+        donation, donation_form = self.random_donation_and_form
+        r = lambda: random.randint(100, 200)
+        random_kwargs = {
+            "slug%d" % r(): "foo-%d" % r(),
+        }
+
+        view = self.post_view
+        view.confirm = False
+        backends = self.get_backend_stub(successful=False)
+        backend_response = backends.get_backend().purchase()
+
+        purchase_failed = fudge.Fake()
+        purchase_failed.expects_call().with_args(backend_response,
+                **random_kwargs)
+
+        with fudge.patched_context(views, "backends", backends):
+            with fudge.patched_context(view, "purchase_failed",
+                    purchase_failed):
+                view.form_is_valid(donation_form, **random_kwargs)
+
+        fudge.verify()
+
+    def test_purchase_failed_passes_kwargs_to_get_context_data(self):
+        backend_response = {
+            "reason": "Some Random Reason",
+            "response": "Some Random Response",
+        }
+        r = lambda: random.randint(100, 200)
+        random_kwargs = {
+            "slug%d" % r(): "foo-%d" % r(),
+        }
+
+        get_context_data = fudge.Fake()
+        (get_context_data.expects_call()
+                .with_args(**random_kwargs)
+                .returns({}))
+
+        view = self.post_view
+        with fudge.patched_context(view, "get_context_data", get_context_data):
+            view.purchase_failed(backend_response, **random_kwargs)
+
+        fudge.verify()
+
 
 def form_is_valid_response(confirmed=False):
     def outer(func):
@@ -168,21 +265,6 @@ def form_is_valid_response(confirmed=False):
 class DonationFormViewPostWithConfirmTestCase(BaseDonationFormViewTestCase):
     view_name = "donations_form_confirm"
 
-    def get_fake_post_request(self, confirmed=False):
-        d = {} if not confirmed else {"confirmed": u"1"}
-        return self.factory.post(self.url, d)
-
-    @property
-    def fake_get_request(self):
-        return self.factory.get(self.url)
-
-    def get_post_view(self, confirmed=False):
-        v = views.DonationFormView(confirm=True)
-        v.request = self.get_fake_post_request(confirmed=confirmed)
-        return v
-
-    post_view = property(get_post_view)
-
     def test_use_confirm_template_false_by_default(self):
         v = views.DonationFormView()
         self.assertFalse(v.use_confirm_template)
@@ -242,6 +324,50 @@ class DonationFormViewPostWithConfirmTestCase(BaseDonationFormViewTestCase):
     def test_redirects_on_confirmed(self, r):
         self.assertIsA(r, HttpResponseRedirect)
 
+    def test_form_is_invalid_receives_kwargs_from_post(self):
+        r = lambda: random.randint(100, 200)
+        random_kwargs = {
+            "slug%d" % r(): "foo-%d" % r(),
+        }
+
+        donation_form = fudge.Fake()
+        donation_form.provides("is_valid").returns(False)
+        get_donation_form = fudge.Fake()
+        get_donation_form.is_callable().returns(donation_form)
+
+        form_is_invalid = fudge.Fake()
+        form_is_invalid.expects_call().with_args(**random_kwargs)
+
+        view = self.post_view
+        with fudge.patched_context(view, "get_donation_form",
+                get_donation_form):
+            with fudge.patched_context(view, "form_is_invalid",
+                    form_is_invalid):
+                view.post({}, **random_kwargs)
+
+        fudge.verify()
+
+    def test_post_passes_kwargs_to_form_is_valid(self):
+        r = lambda: random.randint(100, 200)
+        random_kwargs = {
+            "slug%d" % r(): "foo-%d" % r(),
+        }
+        donation_form = fudge.Fake()
+        donation_form.provides("is_valid").returns(True)
+        get_donation_form = fudge.Fake()
+        get_donation_form.is_callable().returns(donation_form)
+
+        form_is_valid = fudge.Fake()
+        form_is_valid.expects_call().with_args(donation_form=donation_form,
+                **random_kwargs)
+        view = self.post_view
+        with fudge.patched_context(view, "get_donation_form",
+                get_donation_form):
+            with fudge.patched_context(view, "form_is_valid", form_is_valid):
+                view.post({}, **random_kwargs)
+
+        fudge.verify()
+
 
 class DonationFormViewPostTestCase(BaseDonationFormViewTestCase):
     @property
@@ -362,13 +488,6 @@ class DonationFormViewPostTestCase(BaseDonationFormViewTestCase):
         response = self.client.post(self.url, data)
         self.assertRedirects(response, reverse("donations_thanks"))
 
-    def test_displays_errors_on_donor_validation_error(self):
-        data = self.random_post_data
-        del data["first_name"]
-        response = self.client.post(self.url, data)
-        self.assert_template("armstrong/donations/donation.html", response)
-        self.assert_form_has_errors(response, "donor_form", ["first_name", ])
-
     def test_displays_error_on_donation_form_validation_error(self):
         data = self.random_post_data
         del data["ccv_code"]
